#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/rtxSOAP.h"
#include "rtxsrc/rtxSocket.h"
#include "rtxsrc/rtxStreamMemory.h"
#include "rtxsrc/rtxStreamSocket.h"
#include "rtxsrc/rtxStreamFile.h"
#include "CalcWSDL.h"

#define DEFAULT_URL "http://127.0.0.1:9123"

void doAdd (Add_Input*, Add_Output*);
void doSubtract (Subtract_Input*, Subtract_Output*);

int main (int argc, char** argv)
{
   OSCTXT       soapCtxt, decCtxt, encCtxt;
   int          i, stat;
   OSOCTET*     ptmpbuf = 0;
   OSSOAPCONN   soapConn;
   OSIPADDR     ipAddr = OSIPADDR_LOCAL;
   OSBOOL       verbose = FALSE, trace = TRUE;
   OSRTSOCKET   listenSocket;
   const char*  url = DEFAULT_URL;   

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-url")) {
            url = argv[++i];
         }
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: server [-v] [-url <url>] [-notrace]\n");
            printf ("   -v    verbose mode: print trace info\n");
            printf ("   -url  bind URL http://host:port\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Init context */

   stat = rtXmlInitContext (&soapCtxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&soapCtxt);
      return stat;
   }

   if (verbose) rtxSetDiag (&soapCtxt, 1);

   /* --- Request message --- */

   rtxSoapInitConn (&soapConn, &soapCtxt, OSSOAPV1, url);

   /* One-time socket package initialization (required for Windows only) */

   stat = rtxSocketsInit();
   if (0 != stat) {
      printf ("Socket initialization failed.\n");
      rtxErrSetData (&soapCtxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&soapCtxt);
      return stat;
   }
   
   /* Create listener socket */

   if ((stat = rtxSocketCreate (&listenSocket)) != 0) {
      rtxErrSetData (&soapCtxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&soapCtxt);
      return stat;
   }

   /* Bind the socket */

   if ((stat = rtxSocketStrToAddr (soapConn.host, &ipAddr)) != 0) {
      rtxErrSetData (&soapCtxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&soapCtxt);
      return stat;
   }
   if ((stat = rtxSocketBind (listenSocket, ipAddr, soapConn.port)) != 0) {
      rtxErrSetData (&soapCtxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&soapCtxt);
      return stat;
   }

   /* Wait for connection */

   if ((stat = rtxSocketListen (listenSocket, 1)) != 0) {
      rtxErrSetData (&soapCtxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&soapCtxt);
      return stat;
   }

   if (trace) printf ("Wait for connection...\n");

   stat = rtxSoapAcceptConn (listenSocket, &soapConn);
   if (0 != stat) {
      rtxErrSetData (&soapCtxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&soapCtxt);
      return stat;
   }

   /* Wait for request */

   stat = rtxSoapRecvHttp (&soapConn);
   if (stat != 0) {
      rtxErrPrint (&soapCtxt);
      return stat;
   }
   if (trace) printf ("HTTP request received\n");

   /* Read HTTP request into memory.  This is the XML message to be     */
   /* decoded..                                                         */

   stat = rtxSoapRecvHttpContent (&soapConn, &ptmpbuf);
   if (stat != 0) {
      rtxErrPrint (&soapCtxt);
      return stat;
   }   

   stat = rtXmlInitContext (&decCtxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&decCtxt);
      return stat;
   }

   if (verbose) 
      rtxSetDiag (&decCtxt, 1);

   stat = rtxStreamMemoryAttach 
      (&decCtxt, ptmpbuf, soapConn.contentLength, OSRTSTRMF_INPUT);

   if (0 != stat) {
      printf ("rtxStreamMemoryAttach failed\n");
      rtxErrPrint (&decCtxt);
      return stat;
   }

   /* Init encode context */
   stat = rtXmlInitContext (&encCtxt);
   if (0 != stat) {
      printf ("Encode context initialization failed.\n");
      rtxErrPrint (&encCtxt);
      return stat;
   }
   if (verbose) rtxSetDiag (&encCtxt, 1);

   /* Decode */
   printf ("Action: %s\n", soapConn.soapAction);
   if (!strcmp(soapConn.soapAction, "\"http://Calc.com/Calc/Add\"")) {      
      Add_Input request;
      Add_Output response;
      soapConn.soapAction = 0;
      stat = XmlD_Add_Input (&decCtxt, &request);

      if (0 == stat) {
         printf ("Decoding of Request message was successful\n");
         Print_Add_Input("request", &request);
      }
      else {
         printf ("decode failed\n");
         rtxErrPrint (&decCtxt);
         return(1);
      }

      rtxStreamClose (&decCtxt);
      rtxFreeContext (&decCtxt);

      /* --- Response message --- */
      Init_Add_Output (&encCtxt, &response);
      doAdd (&request, &response);

      stat = rtXmlSetEncBufPtr (&encCtxt, 0, 0);

      if (0 == stat)
         stat = XmlE_Add_Output (&encCtxt, &response);

      if (0 != stat) {
         printf ("Encoding failed\n");
         rtxErrPrint (&encCtxt);
         return stat;
      }
   }
   else if /* Received Subtract message */
      (!strcmp(soapConn.soapAction, "\"http://Calc.com/Calc/Subtract\"")) {
      Subtract_Input request;
      Subtract_Output response;
      soapConn.soapAction = 0;
      stat = XmlD_Subtract_Input (&decCtxt, &request);
      if (0 == stat) {
         printf ("Decoding of Request message was successful\n");
         Print_Subtract_Input("request", &request);
      }
      else {
         printf ("decode failed\n");
         rtxErrPrint (&decCtxt);
         return(1);
      }

      rtxStreamClose (&decCtxt);
      rtxFreeContext (&decCtxt);

      /* --- Response message --- */
      Init_Subtract_Output (&encCtxt, &response);
      doSubtract (&request, &response);

      stat = rtXmlSetEncBufPtr (&encCtxt, 0, 0);
   
      if (0 == stat)
         stat = XmlE_Subtract_Output (&encCtxt, &response);

      if (0 != stat) {
         printf ("Encoding failed\n");
         rtxErrPrint (&encCtxt);
         return stat;
      } 
   }

   if (0 != encCtxt.buffer.data) {
      /* Send response */
      soapConn.responseStatusCode = 200;
      stat = rtxSoapSendHttpResponse (&soapConn, rtXmlGetEncBufPtr(&encCtxt));
      if (stat != 0) {
         rtxErrPrint (&soapCtxt);
         return stat;
      }
      if (trace) printf ("HTTP response sent\n");
   }
   rtxFreeContext (&encCtxt);

   /* Close connection */

   rtxStreamClose (&soapCtxt);
   rtxFreeContext (&soapCtxt);

   return stat;
}

void doAdd (Add_Input* req, Add_Output* resp) 
{
   resp->Result = 0.0;
   resp->Result += req->Operands.a + req->Operands.b;
   if (req->Operands.m.cPresent)
      resp->Result += req->Operands.c;
   if (req->Operands.m.dPresent)
      resp->Result += req->Operands.d;
   if (req->Operands.m.ePresent)
      resp->Result += req->Operands.e;
}

void doSubtract (Subtract_Input* req, Subtract_Output* resp)
{
   resp->Result = req->Operands.a;
   resp->Result -= req->Operands.b;
   if (req->Operands.m.cPresent)
      resp->Result -= req->Operands.c;
   if (req->Operands.m.dPresent)
      resp->Result -= req->Operands.d;
   if (req->Operands.m.ePresent)
      resp->Result -= req->Operands.e;

}
