/* This test driver program encodes a data record and writes the        */
/* encoded result to an output file..                                   */

#include <stdio.h>
#include <stdlib.h>
#include "test.h"
#include "rtxmlsrc/osrtxml.h"

#define MAXMSGLEN 4096

int main (int argc, char** argv)
{
   OSCTXT	ctxt;
   OSOCTET	msgbuf[MAXMSGLEN];
   OSXMLSTRING  strdata;
   int		i, stat;
   const char*  filename = "message.json";
   OSBOOL       verbose = FALSE;
   FILE*        fp;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else {
            printf ("usage: writer [ -v ] [ -o <filename> ]\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            return 0;
         }
      }
   }

   /* Init context */

   stat = rtxInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }
   rtxSetDiag (&ctxt, verbose);

   /* Populate value to be encoded */

   strdata.cdata = FALSE;
   strdata.value = OSUTF8("Hello World");

   /* Encode */

   stat = rtxInitContextBuffer (&ctxt, msgbuf, sizeof(msgbuf));
   if (stat != 0) { 
      printf ("Context buffer initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   ctxt.buffer.data[0] = '\0';
   rtxCtxtSetFlag (&ctxt, OSDISSTRM); /* disable stream operations */
   ctxt.state = OSXMLINIT;

   if (0 == stat) {
      stat = JsonEnc_hello (&ctxt, &strdata);
   }

   if (0 == stat) {
      printf ("encoded message:\n");
      puts ((char*)msgbuf);
      printf ("\n");
   }
   else {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
      return stat;
   }
  
   /* Write the encoded message out to the output file */

   if ((fp = fopen (filename, "w")) != NULL) {
      fputs ((char*)msgbuf, fp);
      fclose (fp);
   }
   else {
      printf ("Error opening %s for write access\n", filename);
      return -1;
   }

   rtxFreeContext (&ctxt);
   return 0;
}
