/* This test driver program encodes a data record and writes the        */
/* encoded result to an output file..                                   */

#include <stdio.h>
#include <stdlib.h>
#include "prod.h"
#include "rtxsrc/rtxDiag.h"
#include "rtxsrc/rtxFile.h"

#define MAXMSGLEN 1024

int main (int argc, char** argv)
{
   ProductType  product;
   OSCTXT	ctxt;
   OSOCTET	msgbuf[MAXMSGLEN];
   int		i, stat;
   const char*  filename = "message.json";
   OSBOOL       verbose = FALSE;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else {
            printf ("usage: writer [ -v ] [ -o <filename>\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            return 0;
         }
      }
   }

   /* Init context */

   stat = rtxInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }
   if (verbose)
      rtxSetDiag (&ctxt, 1);

   /* Populate structure of generated type */

   Init_ProductType (&ctxt, &product);
   product.number = 222;
   product.size = 10;
   product.effDate.value = (const OSUTF8CHAR*)"2002-02-25";
   product.m.effDatePresent = TRUE;

   /* Encode */

   stat = rtxInitContextBuffer (&ctxt, msgbuf, sizeof(msgbuf));

   if (0 == stat)
      stat = JsonEnc_product (&ctxt, &product);

   if (0 == stat) {
      printf ("encoded message:\n");
      puts ((char*)msgbuf);
      printf ("\n");
   }
   else {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
      return stat;
   }
  
   /* Write encoded message out to the output file */

   stat = rtxFileWriteText (filename, (const char*)msgbuf);
   if (0 != stat) {
      printf ("Error writing to file %s\n", filename);
   }

   rtxFreeContext (&ctxt);

   return stat;
}
