#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/OSRTFileInputStream.h"
#include "rtxmlsrc/rtXmlCppMsgBuf.h"
#include "test.h"
#include "rtxsrc/rtxMemLeakCheck.h"

DECLARE_MEMLEAK_DETECTOR;

int main (int argc, char** argv)
{
   int          i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;

   // Process command line arguments

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-i")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: reader [-v] [-i <filename>] [-notrace]\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -i <filename>  read encoded msg from <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   // Create buffer and control objects

   Catalogue_ELEM value;
   OSRTFileInputStream in (filename);
   OSXMLDecodeBuffer decodeBuffer (in);

   if (decodeBuffer.getStatus ()) {
      decodeBuffer.printErrorInfo();
      return decodeBuffer.getStatus ();
   }

   Catalogue_CC pdu (decodeBuffer, value);

   decodeBuffer.setDiag (verbose);

   // Validate

   in.mark (INT_MAX);
   stat = pdu.validate();
   if (0 == stat) {
      if (trace) {
         printf ("Validated successfully\n\n");
      }
   }
   else {
      printf ("Validation failed\n");
      decodeBuffer.printErrorInfo();
   }

   in.reset ();
   decodeBuffer.resetErrorInfo ();
   stat = 0;

   // Decode

   stat = pdu.decode();

   if (0 == stat) {
      if (trace) {
         printf ("decoded XML message:\n");
         pdu.print ("Catalogue");
         printf ("\n");
      }
   }
   else {
      printf ("Decoding failed\n");
      decodeBuffer.printErrorInfo();
      return stat;
   }

   return 0;
}
