import com.test.*;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.io.ByteArrayOutputStream;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;
import java.util.List;
import com.objsys.xbinder.runtime.*;

/**
 * This test program encodes a data record and writes the encoded result to an output file.
 */
public class Writer {

   private static DatatypeFactory dtFactory;

   static {
      try {
         dtFactory = DatatypeFactory.newInstance();
      }
      catch ( javax.xml.datatype.DatatypeConfigurationException dce ) 
      { throw new RuntimeException(dce); }
   }

   private static String outputFile = "message.xml";
   private static String charset = "UTF-8";     //default is UTF-8
   private static boolean trace = true;

   public static void main(String args[]) throws Exception 
   {
      //process command line to get the name of the output file
      //process command line to get the name of the input file
      for(int arg = 0; arg < args.length; arg++) {
         if ( args[arg].equals("-o")) {
           outputFile = args[++arg];
         }
         else if ( args[arg].equals("-notrace" ) ) trace = false;
         else {
            System.out.println("Usage: java Writer [-o <filename>] [-notrace]");
            System.out.println("  -o <filename> specifies the output file");
            System.out.println("  -notrace turns off echoing encoded data to stdout");
            System.exit(1);
         }
      }

      //create _CC object
      Catalogue_CC root = new Catalogue_CC();

      Catalogue_ELEM catalogue = new Catalogue_ELEM();
      root.setCatalogue(catalogue);

      java.util.List<_PublicationSG> publication = catalogue.getPublication();

      _PublicationSG publicationN;

      XMLGregorianCalendar date;
      java.util.List<String> author;

      BookType book;
      MagazineType magazine;

      publicationN = new _PublicationSG();
      publication.add(publicationN);
      book = new BookType();
      publicationN.setBook(book);
      book.setTitle("Illusions The Adventures of a Reluctant Messiah");
      author = book.getAuthor();
      author.add("Richard Bach");
      date = dtFactory.newXMLGregorianCalendarDate(1977, DatatypeConstants.FIELD_UNDEFINED, DatatypeConstants.FIELD_UNDEFINED, DatatypeConstants.FIELD_UNDEFINED);
      book.setDate(date);
      book.setISBN("0-440-34319-4");
      book.setPublisher("Dell Publishing Co.");

      publicationN = new _PublicationSG();
      publication.add(publicationN);
      magazine = new MagazineType();
      publicationN.setMagazine(magazine);
      magazine.setTitle("Natural Health");
      date = dtFactory.newXMLGregorianCalendarDate(1999, DatatypeConstants.FIELD_UNDEFINED, DatatypeConstants.FIELD_UNDEFINED, DatatypeConstants.FIELD_UNDEFINED);
      magazine.setDate(date);

      publicationN = new _PublicationSG();
      publication.add(publicationN);
      book = new BookType();
      publicationN.setBook(book);
      book.setTitle("The First and Last Freedom");
      author = book.getAuthor();
      author.add("J. Krishnamurti");
      date = dtFactory.newXMLGregorianCalendarDate(1954, DatatypeConstants.FIELD_UNDEFINED, DatatypeConstants.FIELD_UNDEFINED, DatatypeConstants.FIELD_UNDEFINED);
      book.setDate(date);
      book.setISBN("0-06-064831-7");
      book.setPublisher("Harper & Row");


      //Encode data
      ByteArrayOutputStream bos = new ByteArrayOutputStream(32768);

      XBXmlEncoder encoder;
      encoder = new XBXmlEncoder(bos, charset);
      encoder.setNamespaces(_test.namespaceContext);
      root.encodeDocument(encoder);
      encoder.close();

      if ( trace ) {
         //write encoded data to System.out
         System.out.println("Encoded data:\n");
         bos.writeTo(System.out);
         System.out.println();
      }

      //write encoded data to output file
      FileOutputStream fout = new FileOutputStream(outputFile);
      bos.writeTo(fout);
      fout.close();
   }
}
